


/**
 * animate dealing of two cards to each player at beginning of round.
 *
 * player: array of all player objects.
 *
 */
function animateDealingStartHands(player, callback) {
    var currentPlayer = -1;
    var card_number = 1; // counts how many cards have been dealt per player
    temp();

    function temp() {
        if (currentPlayer >= player.length - 1) { // all player have gotten a card
            card_number++; // deal second card for each player
            currentPlayer = -1; // start with first player
        }

        if (card_number > 2) { // two cards per player have been dealt
            callback();
        } else { // there are cards left to be dealt
            currentPlayer++;
            animateCardDeal(player[currentPlayer].name, function() { // animate this card deal
                draw(document.getElementById(player[currentPlayer].name).getContext("2d"), player[currentPlayer].hand.cards[0] , 0, 0); // display card after deal-animation is finished for this card
                temp(); // show next deal-animation
            });
        }
    }
}

function animateSplit(currentPlayer, splitPlayer, callback) {
   draw(document.getElementById(currentPlayer.name).getContext("2d"), currentPlayer.hand.cards.getAllCards()[0] , 0, 0); // display card that is left for orig. player
   animateCardDealFromTo(currentPlayer.name + "d", splitPlayer.name + "d", splitPlayer.hand.cards.getAllCards()[0], // animate card move (split)
        function () {
            // display the two split cards            
            draw(document.getElementById(splitPlayer.name).getContext("2d"), splitPlayer.hand.cards.getAllCards()[0], 0, 0); // display card that is new for split player
            animateCardDeal(currentPlayer.name, // animate deal to first player
                function () {
                    animateCardDeal(splitPlayer.name, // animate deal to second player
                        function () {
                            callback();
                        }
                    );
                }
            );
        }
        );
}


var animCanvasDiv; // use the same canvas for all dealt cards.
var animationRunning = false; // flag to check if currently an animation is running
/*
 * animates the dealing of one card to the player with the given name.
     * playername: string, name of the player that the card should be delt.
     * callback: function, show card function which will be executed after
     *  animation is finished.
     */
function animateCardDeal(playername, callback) {
    animateCardDealFromTo("dealerd", playername + "d", new deck.Card(5, deck.ColorEnum.HEARTS), callback);
}

function animateCardDealFromTo(playernameFrom, playerNameTo, card, callback) {
    if (animationRunning) {
        callback(); // callback in case player presses new game while dealing animationis running, display correct cards.
        return; // as canvas is reused, only one animation can be displayed at the time.
    // if it becomes necessary to display more, use the old animateCardDeal below.
    }
    var dealerdivX = document.getElementById(playernameFrom).style.left;
    var dealerdivY = document.getElementById(playernameFrom).style.top;

    var playerdivX = parseInt(document.getElementById(playerNameTo).style.left);
    var playerdivY = parseInt(document.getElementById(playerNameTo).style.top);

    if (typeof animCanvas == "undefined") { // init canvas
        // temp element to draw card on
        animCanvasDiv = document.createElement( "div" );
        animCanvasDiv.className = "semicircle";
        animCanvasDiv.style.left = dealerdivX;
        animCanvasDiv.style.top = dealerdivY;
        animCanvasDiv.id = "animationd";

        var animCanvas = document.createElement( "canvas" );
        animCanvas.id = "animation";
        animCanvas.width = cardWidth * 5;
        animCanvas.height = cardHeight;
        draw(animCanvas.getContext("2d"), card , 0, 0);
        animCanvasDiv.appendChild(animCanvas);
        document.getElementById("blackjack-table").appendChild(animCanvasDiv);
    } else {
        draw(animCanvas.getContext("2d"), card , 0, 0);
        animCanvasDiv.style.left = dealerdivX;
        animCanvasDiv.style.top = dealerdivY;
        animCanvasDiv.style.display = 'inline'; // set canvas visible
    }

    animate();

    function animate() { // TODO this needs speeding up.
        animationRunning = true;
        var oldX = parseInt(animCanvasDiv.style.left); // TODO these do not need to be parsed each time. use other variables.
        var oldY = parseInt(animCanvasDiv.style.top);
        if (Math.abs(oldY - playerdivY) < 15 && Math.abs(oldX - playerdivX) < 15) {
            cleanup();
            callback();
        } else {
            var speed = 12; // distance card moves in one turn
//                            var rotation = toDegree(Math.atan2(playerdivY - oldY, playerdivX - oldX)) + 90;
//                            xDirection = Math.sin(toRadian(rotation)) * speed;
//                            yDirection = Math.cos(toRadian(rotation)) * -speed;

            // optimisation:
            var rotation = Math.atan2(playerdivY - oldY, playerdivX - oldX) + 0.5 * Math.PI;
            var xDirection = Math.sin(rotation) * speed;
            var yDirection = Math.cos(rotation) * -speed;

            animCanvasDiv.style.left = oldX + xDirection +'px';
            animCanvasDiv.style.top = oldY + yDirection +'px';

            setTimeout(animate,10); // time between turns in ms
        }
    }

//            function toRadian(degree) {
//                return degree * (Math.PI/180);
//            }
//
//            function toDegree(radian) {
//                return radian * (180/Math.PI);
//            }

    function cleanup() {
        animationRunning = false;
        animCanvasDiv.style.display = 'none'; // set canvas invisible
    }
}


// old animateCardDeal. uses new canvas for each dealt card.
//    /*
//     * playerNumber: int, number of the player that the card should be delt.
//     * callback: function, show card function which will be executed after
//     *  animation is finished.
//     */
//    function animateCardDeal(playerNumber, callback) {
//        var dealerdivX = document.getElementById("dealerd").style.left;
//        var dealerdivY = document.getElementById("dealerd").style.top;
//
//        var playerdivX = parseInt(document.getElementById("player" + playerNumber + "d").style.left);
//        var playerdivY = parseInt(document.getElementById("player" + playerNumber + "d").style.top);
//
//        // temp element to draw card on
//        var div = document.createElement( "div" );
//        div.className = "semicircle";
//        div.style.left = dealerdivX;
//        div.style.top = dealerdivY;
//        div.id = "animationd";
//
//        var canvas = document.createElement( "canvas" );
//        canvas.id = "animation";
//        canvas.width = cardWidth * 5;
//        canvas.height = cardHeight;
//        div.appendChild(canvas);
//        document.getElementById("blackjack-table").appendChild(div);
//
//        draw(canvas.getContext("2d"), new deck.Card(5, deck.ColorEnum.HEARTS) , 0, 0);
//        animate();
//
//        function animate() { // TODO this needs speeding up.
//            var oldX = parseInt(div.style.left);
//            var oldY = parseInt(div.style.top);
//            if (Math.abs(oldX - playerdivX) < 25 && Math.abs(oldY - playerdivY) < 25) {
//                cleanup();
//                callback();
//            } else {
//                var speed = 12; // distance card moves in one turn
//                var rotation = toDegree(Math.atan2(playerdivY - oldY, playerdivX - oldX)) + 90;
//                xDirection = Math.sin(toRadian(rotation)) * speed;
//                yDirection = Math.cos(toRadian(rotation)) * -speed;
//
//                div.style.left = oldX + xDirection +'px';
//                div.style.top = oldY + yDirection +'px';
//
//                setTimeout(animate,10); // time between turns in ms
//            }
//        }
//
//        function toRadian(degree) {
//            return degree * (Math.PI/180);
//        }
//
//        function toDegree(radian) {
//            return radian * (180/Math.PI);
//        }
//
//        function cleanup() {
//        // TODO this sometimes fails. either find out why, or reuse the same canvas all the time and hide it when it is not in use
//            document.getElementById("blackjack-table").removeChild(document.getElementById("animationd"));
//        }
//    }