/*global deck.js*/
var jack = namespace('SoftwareTalk.CardGames.BlackJack');

/*
 * a blackjack hand.
 */
jack.Hand = function (){ // TODO Hand should extend Deck. but js extend does not work very well...
    this.cards = new deck.Deck(false); // Hand starts as empty deck.
    this.hard = true;
    this.bet = 0; // amount of cash bet on this hand.
}
// delecation methods
jack.Hand.prototype.putCard = function (card) {return this.cards.putCard(card);}
jack.Hand.prototype.getCard = function (card) {return this.cards.getCard();}
jack.Hand.prototype.toString = function () {
    var val = this.handValue();
    var hard = this.isHard() ? 'hard' : 'soft';
    return "" + hard + " " + val + " (" + this.cards.toString() + ')';
}
/*
 * returns true if this is a hard hand (a hand is soft if it has an ace that is counted as 11. otherwise it is hard).
 * this value is set by the function handValue() so handValue() MUST be called first.
 */
jack.Hand.prototype.isHard = function () {
    return this.hard;
}
jack.Hand.prototype.isPair = function () {
    function isTentoKing(card) {
        return ((new Array(10, 11, 12, 13)).indexOf(card.value) >= 0);
    }
    if (this.cards.cardsInDeck() != 2) {
        return false; // more than 2 cards cannot be pairs.
    }
    var mycards = this.cards.getAllCards();
    // 10, jack, queen and king are equal:
    if (isTentoKing(mycards[0]) && isTentoKing(mycards[1])) {
        return true;
    } else {
        return (isTentoKing(mycards[0]) && isTentoKing(mycards[1])) // both are between 10 and king
            || mycards[0].value == mycards[1].value; // or both have equal value
    }
}
jack.Hand.prototype.isBlackJack = function () {
    if (this.cards.cardsInDeck() != 2) {
        return false;
    }
    var mycards = this.cards.getAllCards();
    if (mycards[0].value == 14 // first card ace
        && ((new Array(10, 11, 12, 13)).indexOf(mycards[1].value) >= 0) // second card 10, jack, queen or king
        ) {
        return true;
    }
    if (mycards[1].value == 14 // second card ace
        && ((new Array(10, 11, 12, 13)).indexOf(mycards[0].value) >= 0) // second card 10, jack, queen or king
        ) {
        return true;
    }
    return false
}
jack.Hand.prototype.handValue = function () {
    this.hard = true;
    var card_array = this.cards.getAllCards(); // return all cards in hand as array without removing them
    var value = 0;
    var amountAces = 0;
    for( var i=0, l=card_array.length; i<l; i++ ) {
        var card = card_array[i];
        if (card.value >= 2 && card.value <= 10) { // add value for number cards
            value += card.value;
        } else { // add value for picture cards except aces
            switch(card.value) {
                case 11: // jack
                    value += 10;
                    break;
                case 12: // queen
                    value += 10;
                    break;
                case 13: // king
                    value += 10;
                    break;
                case 14: // ace
                    amountAces++;
                    break;
                case 15: // joker
                    value += 0; // joker should not be in blackjack
                    break;
                default:
                    value += 0; // invalid card
            }
        }
    }
    for (var i=1; i <= amountAces; i++) { // add value for aces
        if (value + 11 > 21) { // if ace is counted as 11 leads to bust, use 1
            value += 1;            
        } else {
            this.hard = false;
            value += 11; // else use 11
        }
    }
    return value;
}

jack.Player = function (name, money, isReal, belongsTo) {
    this.name = name;
    this.money = money;

    this.hand = new jack.Hand();
    this.isstand = false;
    this.lastWinningsMessage = ""; // stores the winning message so the view can print it. at some point refactor this...

    // code for split action:
    // (for each split, a new "dummy" player will be added which will be removed at the end of the round. (this is not so good, but i forgot about split when writing the game at first...)
    this.isReal = isReal; // is player an actual player or just a temporary one for split action?
    this.belongsTo = belongsTo; // the name of the player this dummy player belongs to
    this.amountChildren = 0; // how many dummy player this player has
}
/*
 * deal a new hand consisting of two cards to this player (to be called on new round).
 */
jack.Player.prototype.deal = function (card1, card2) {
    this.isstand = false;
    this.hand = new jack.Hand();
    card1.showCard(); // player cards are never hidden
    card2.showCard();
    this.hand.putCard(card1);
    this.hand.putCard(card2);
}
/**
 * adds the given amount of money to this player.
 * If this player is a dummy player resulting from a split, the money will be
 * added to the original player.
 */
jack.Player.prototype.addMoney = function (amountmoney, game) {
    if (this.name == this.belongsTo) {
        this.money += amountmoney;
    } else { // split player
        game.getPlayer(this.belongsTo).money += amountmoney;
    }
}
/*
 * add given card to current hand of player.
 */
jack.Player.prototype.hit = function (card1) {
    this.isstand = false;
    card1.showCard(); // player cards are never hidden
    this.hand.putCard(card1);
}
jack.Player.prototype.stand = function () {
    this.isstand = true;
}
jack.Player.prototype.toString = function () {return this.name + ": " + "$" + this.money + ": " + this.hand.toString();}


jack.dealerRulesEnum = {
    H17 : {
        value: 1, 
        name: "H17"
    }, // hit soft 17
    S17 : {
        value: 2, 
        name: "S17"
    }  // stand on soft 17
};
jack.Dealer = function (dealerRules) {
    this.hand = new jack.Hand();
    this.isstand = false;
    this.dealerRules = dealerRules;
}
jack.Dealer.prototype.deal = function (card1, card2) {
    this.isstand = false;
    this.hand = new jack.Hand();
    card1.showCard(); // first card is shown
    // second is not
    this.hand.putCard(card1);
    this.hand.putCard(card2);
}
jack.Dealer.prototype.hit = function (card1) {
    this.isstand = false;
    this.hand.putCard(card1);
}
jack.Dealer.prototype.stand = function () {
    this.isstand = true;
}
jack.Dealer.prototype.toString = function () {return this.hand.toString();}


jack.Game = function (numberOfPlayers, dealerRules, numberOfDecks) {
    this.player = new Array();
    this.dealer = new jack.Dealer(dealerRules);
    this.gdeck = new deck.Deck(false);//start with empty deck
    this.numberOfDecks = numberOfDecks;
    this.currentPlayer = -1; // player whos turn it is. -1 for no players turn.
    for (var i = 0; i < numberOfPlayers; i++) {
        this.player[i] = new jack.Player("player" + i, 100, true, "player" + i);
    }
}
/*
 * call this to start a new round.
 * a new deck will be created, each player will be dealt two cards, and the dealer will
 * be dealt as well.
 */
jack.Game.prototype.initRound = function () {
    this.gdeck = deck.getMultiDeck(this.numberOfDecks);
    printDebug('DEALING to ' + this.player.length + ' players');

    // remove all temp players that resulted from a split action
    for (var i = this.player.length -1; i >= 0; i--) {
        if (!this.player[i].isReal) {
            this.player.splice(i, 1);
        } else {
            this.player[i].amountChildren = 0; // reset amount of children for real player
        }
    }

    // deal starting hand to each player
    for(var i = 0; i < this.player.length; i++) {
        this.player[i].deal(this.gdeck.getCard(), this.gdeck.getCard()); // check if card is not null
        printDebug('' + this.player[i]);
    }
    this.dealer.deal(this.gdeck.getCard(), this.gdeck.getCard());
    printDebug('Dealer: ' + this.dealer);
    this.currentPlayer = 0;
    printDebug('PLAYER ' + this.currentPlayer + ' TURN');
}
jack.Game.prototype.getPlayer = function (playername) {
    for(var i = 0; i < this.player.length; i++) {
        if (this.player[i].name == playername) {
            return this.player[i];
        }
    }
}
/*
 * call this after all players submited their input (meaning at the end of a round).
 * The dealer will play its hand.
 */
jack.Game.prototype.playDealer = function () {
    printDebug('DEALERS TURN ' + this.dealer);
        // dealer hits until 17, then stops.
        printDebug('' + this.dealer);

        // TODO recheck this while loop.
        while(this.dealer.hand.handValue() < 17 // hit until reaching 17 or more
            || (this.dealer.dealerRules == jack.dealerRulesEnum.H17 && this.dealer.hand.handValue() == 17 && !this.dealer.hand.isHard())) // if rules say so, also hit soft 17
        {
            this.dealer.hit(this.gdeck.getCard());
            printDebug('' + this.dealer);
        }
        this.dealer.stand();
}
jack.Game.prototype.showDealerCards = function () {
    var cards = this.dealer.hand.cards.getAllCards();
    for (var i = 0; i < cards.length; i++) {
        cards[i].showCard();
    }
}
jack.Action = {
  HIT : {value: 1, name: "Hit"},
  STAND : {value: 2, name: "Stand"},
  DOUBLE : {value: 3, name: "Double"},
  SPLIT : {value: 4, name: "Split"},
  SURRENDER : {value: 5, name: "Surrender"}
};
/*
 * returns true if current player is last player and finished.
 */
jack.Game.prototype.playCurrentPlayer = function (action) {
        switch(action) {
            case jack.Action.HIT:
                this.player[this.currentPlayer].hit(this.gdeck.getCard());
                if (this.player[this.currentPlayer].hand.handValue() > 21) { // bust
                    this.player[this.currentPlayer].stand();
                    this.currentPlayer++;
                    printDebug('PLAYER ' + this.currentPlayer + ' TURN');
                }
                printDebug('' + this.player[this.currentPlayer]);
                break;
            case jack.Action.STAND:
                this.player[this.currentPlayer].stand();
                this.currentPlayer++;
                printDebug('' + this.player[this.currentPlayer]);
                printDebug('PLAYER ' + this.currentPlayer + ' TURN');
                break;
            case jack.Action.DOUBLE:
                if (this.player[this.currentPlayer].hand.cards.cardsInDeck() == 2) { // only allow double as first action
                    this.player[this.currentPlayer].hand.bet *= 2;
                    this.player[this.currentPlayer].hit(this.gdeck.getCard());
                    this.currentPlayer++;
                    printDebug('PLAYER ' + this.currentPlayer + ' TURN');
                } else {
                    printDebug('double not allowed');
                }
                break;
            case jack.Action.SPLIT:
                var currPlayer = this.player[this.currentPlayer];
                var parent = this.getPlayer(currPlayer.belongsTo);
                parent.amountChildren += 1;

                if (currPlayer.hand.cards.cardsInDeck() == 2 
                    && currPlayer.hand.isPair()
                    && parent.amountChildren < 4) { // only allow split as first action, if player has pair and has split less than 3 times
                    var dummyPlayer = new jack.Player(currPlayer.name + "-dummy" + parent.amountChildren, 0, false, currPlayer.belongsTo);
                    dummyPlayer.hit(currPlayer.hand.cards.getCard()); // give one of the cards to the dummy player
                    dummyPlayer.hand.bet = currPlayer.hand.bet;
                    this.player.splice(this.currentPlayer + 1, 0, dummyPlayer); // insert dummy player after current player

                    currPlayer.hit(this.gdeck.getCard());
                    dummyPlayer.hit(this.gdeck.getCard());       


                } else {
                    printDebug('split not allowed');
                }
                break;
            default:
                printDebug('invalid action');
        }
        if (this.currentPlayer >= this.player.length) {
            this.playDealer();
            this.applyWinnings();
            this.showDealerCards();
            return true;
        }
        return false;
}
jack.Game.prototype.applyWinnings = function () {
    printDebug('RESULTS');    
    var dealerHasBlackJack = this.dealer.hand.isBlackJack();
    var dealerHandValue = this.dealer.hand.handValue();
    for(var i = 0; i < this.player.length; i++) {
        var playerHandValue = this.player[i].hand.handValue();
        if (playerHandValue > 21) { // first, every player that bust loses:            
            this.player[i].addMoney(-this.player[i].hand.bet, this);
            printDebug('PLAYER' + i + ' BUST, bet lost');
            this.player[i].lastWinningsMessage = "Bust! Lost $" + this.player[i].hand.bet;
        } else if (this.player[i].hand.isBlackJack() 
            && this.player[i].amountChildren == 0) { // blackjack is no blackjack after split
            if (!dealerHasBlackJack) { // player has blackjack and dealer does not:
                this.player[i].addMoney(this.player[i].hand.bet * (3/2), this);
                printDebug('PLAYER' + i + ' BLACKJACK, won ' + (this.player[i].hand.bet * (3/2)));
                this.player[i].lastWinningsMessage = "Blackjack! Won $" + (this.player[i].hand.bet * (3/2));
            } else { // else: both have blackjack, player does not lose but also does not win
                printDebug('PLAYER' + i + ' BLACKJACK, DEALER also BLACKJACK, no winnings');
                this.player[i].lastWinningsMessage = "Tie!";
            }
        } else {
            if (dealerHasBlackJack) {
                // we already checked if player has blackjack, so here player does not but dealer does. player loses
                this.player[i].addMoney(-this.player[i].hand.bet, this);
                printDebug('PLAYER' + i + ' LOST (DEALER HAS BLACKJACK): ' + this.player[i].hand.bet);
                this.player[i].lastWinningsMessage = "Lost! $" + this.player[i].hand.bet;
            } else if (dealerHandValue > 21) { // player did not bust, but dealer did. player wins.
                this.player[i].addMoney(this.player[i].hand.bet, this);
                printDebug('PLAYER' + i + ' WON (DEALER BUST): ' + this.player[i].hand.bet);
                this.player[i].lastWinningsMessage = "Won! $" + this.player[i].hand.bet;
            } else if (playerHandValue > dealerHandValue) {
                this.player[i].addMoney(this.player[i].hand.bet, this);
                printDebug('PLAYER' + i + ' WON: ' + this.player[i].hand.bet);
                this.player[i].lastWinningsMessage = "Won! $" + this.player[i].hand.bet;
            } else if (playerHandValue < dealerHandValue) {
                this.player[i].addMoney(-this.player[i].hand.bet, this);
                printDebug('PLAYER' + i + ' LOST: ' + this.player[i].hand.bet);
                this.player[i].lastWinningsMessage = "Lost! $" + this.player[i].hand.bet;
            } else {
                printDebug('PLAYER' + i + ' TIED');
                this.player[i].lastWinningsMessage = "Tie!";
            }
        }
    }
}

    function printDebug(text) {
        var txt = $("textarea#debug");
        txt.val( txt.val() + text + "\n");

        txt.scrollTop(txt[0].scrollHeight - txt.height());
    }