/*global deck.js*/
/*global Blackjack.js*/
/*global CardImages.js*/

// returns best move using basic strategy and following rules:
// 4 or more decks, dealer stands on soft 17, double any two cards, double after split, no surrender, no peek
// src: http://en.wikipedia.org/wiki/Blackjack, http://wizardofodds.com/games/blackjack/strategy/calculator/
function getBestMove(handPlayer, handDealer) {
    var dealerShowing = handDealer.cards.getAllCards()[0];

    var dealerHandValue = getBJCardValue(dealerShowing.value);
    var playerHandValue = handPlayer.handValue();
    var playerIsHard =  handPlayer.cards.cardsInDeck() == 2 || handPlayer.isHard();
    var playerIsPair = handPlayer.cards.cardsInDeck() == 2 && handPlayer.isPair();
    if (playerIsPair) {
        if (playerHandValue == 20) {
            return jack.Action.STAND;
        } else if (playerHandValue == 18 && (dealerHandValue == 7 || dealerHandValue == 10 || dealerHandValue == 11)) {
            return jack.Action.STAND;
        } else if (playerHandValue == 10 && (dealerHandValue != 10 && dealerHandValue != 11)) {
            return jack.Action.DOUBLE; // or hit
        } else if (playerHandValue == 8 && (dealerHandValue >= 2 && dealerHandValue <= 4)) {
            return jack.Action.HIT;
        } else if ((playerHandValue >= 4 && playerHandValue <= 14) && (dealerHandValue >= 7)
            && (!(playerHandValue == 14 && dealerHandValue == 7) && !((playerHandValue == 4 || playerHandValue == 6) && dealerHandValue == 7))
            && !(playerHandValue == 10 && dealerHandValue <= 9)) {
            return jack.Action.HIT;
        } else {
            return jack.Action.SPLIT;
        }
    } else if (playerIsHard) {
        if (playerHandValue >= 17) {
            return jack.Action.STAND;
        } else if (playerHandValue >= 13 && dealerHandValue <= 6) {
            return jack.Action.STAND;
        } else if (playerHandValue == 12 && dealerHandValue <= 6 && dealerHandValue >= 4 ) {
            return jack.Action.STAND;
        } else if (playerHandValue == 16 && (dealerHandValue == 9 || dealerHandValue == 10 || dealerHandValue == 11)) {
//            if (handPlayer.cards.cardsInDeck() == 2) {
//                 return jack.Action.SURRENDER; // or hit
//            } else {
                return jack.Action.HIT;
//            }
        } else if (playerHandValue == 11 && dealerHandValue != 11) {
            return doubleOrHit(handPlayer);
        } else if (playerHandValue == 10 && (dealerHandValue != 11 && dealerHandValue != 10)) {
            return doubleOrHit(handPlayer);
        } else if (playerHandValue == 9 && (dealerHandValue >= 3 && dealerHandValue <= 6)) {
            return doubleOrHit(handPlayer);
        } else {
            return jack.Action.HIT;
        }
    } else { // soft hand
        if (playerHandValue >= 19) {
            return jack.Action.STAND;
        } else if (playerHandValue == 18 && (dealerHandValue == 2 || dealerHandValue == 7 || dealerHandValue == 8)) {
            return jack.Action.STAND;
        } else if (playerHandValue == 18 && (dealerHandValue >= 3 && dealerHandValue <= 6)) {
            if (handPlayer.cards.cardsInDeck() == 2) {
                 return jack.Action.DOUBLE; // or stand
            } else {
                return jack.Action.STAND;
            }   
        } else if (playerHandValue == 17 && (dealerHandValue >= 3 && dealerHandValue <= 6)) {
            return doubleOrHit(handPlayer);
        } else if ((playerHandValue == 15 || playerHandValue == 16) && (dealerHandValue >= 4 && dealerHandValue <= 6)) {
            return doubleOrHit(handPlayer);
        } else if ((playerHandValue == 13 || playerHandValue == 14) && (dealerHandValue == 5 || dealerHandValue == 6)) {
            return doubleOrHit(handPlayer);
        } else {
            return jack.Action.HIT;
        }
    }

    function doubleOrHit(handPlayer) {
            if (handPlayer.cards.cardsInDeck() == 2) {
                 return jack.Action.DOUBLE; // or hit
            } else {
                return jack.Action.HIT;
            } 
    }

}

function getBJCardValue(card) {
    var value = 0;
            switch(card.value) {
                case 11: // jack
                    value = 10;
                    break;
                case 12: // queen
                    value = 10;
                    break;
                case 13: // king
                    value = 10;
                    break;
                case 14: // ace
                    value = 11;
                    break;
                case 15: // joker
                    value = 0; // joker should not be in blackjack
                    break;
                default:
                    value = card.value;
            }
}

