/*global deck.js*/
/*global Blackjack.js*/
/*global CardImages.js*/
/*global Animation.js*/
/*global basicStrategy.js*/
/*global jquery-1.8.3.min.js*/

/*
 * the blackjack view and controller.
 */

$(document).ready(function(){

function hasClass(ele,cls) {
    return ele.className.match(new RegExp('(\\s|^)'+cls+'(\\s|$)'));
}

function removeClass(id,cls) {
        var ele = document.getElementById(id);
        if (hasClass(ele,cls)) {
            var reg = new RegExp('(\\s|^)'+cls+'(\\s|$)');
            ele.className=ele.className.replace(reg,' ');
        }
    }

function addClass(id, myclass) {
  var el = document.getElementById(id);
  if(el) {
    el.className += el.className ? ' ' + myclass : myclass;
  }
}

    // settings
    var numberOfPlayer = 1;
    var animate = true;
    var dealerRules = jack.dealerRulesEnum.H17;
    var numberOfDecks = 1;
    var displayHandValue = false;
    var hintbestmove = false;
    var timeBetweenRounds = 1500;

    function loadSettings() {
                // TODO validate input
        numberOfPlayer = $("#numberOfPlayer").val();
        numberOfDecks = $("#numberOfDecks").val();
        timeBetweenRounds = $("#timeBetweenRounds").val();
        animate = (($('#isanimation:checked').val()) ? true : false);
        displayHandValue = (($('#isdisplayvalue:checked').val()) ? true : false);
        hintbestmove = (($('#showbestmove:checked').val()) ? true : false);
        if ($('input[name=radioName]:checked', '#myForm').val() == "H17") {
            dealerRules = jack.dealerRulesEnum.H17;
        } else {
            dealerRules = jack.dealerRulesEnum.S17;
        }
    }

    function initView() {
    

        // clear old view
        document.getElementById("blackjack-table").innerHTML = '';

        // set new view
        var startAngle = Math.PI / game.player.length,
        angle = startAngle / 2,
        radius = 450,
        offset = window.innerWidth / 2;

        // position the players
        for ( var i = 0; i < game.player.length; i++ ) {
            addPlayerCanvas(
                (radius * Math.cos( angle ) + offset) // arange player in half-circle
                    - ((cardWidth * 5) / 2) // position cards in middle
                    + "px",
                (radius * Math.sin( angle )) // arange player in half-circle    // TODO split view can be realised by adding 260 to this and keeping the rest as is.
                    + "px",
                game.player[i].name);
            angle += startAngle;
        }

        // position the dealer
        addPlayerCanvas(
            (1 * Math.cos( startAngle ) + offset )
                - ((cardWidth * 5) / 2) // position cards in middle
                + "px",
            (radius / 4) + "px",
            "dealer");        

        // control buttons and labels
        for (var j = 0; j < game.player.length; j++) {
            addPlayerControl(game.player[j].name, true);
        }

        // hand value label for dealer
        if (displayHandValue) {
            var dealerdiv = document.getElementById("dealerd");
            addNewlineTo(dealerdiv);
            dealerdiv.innerHTML += "Value: ";
            dealerdiv.appendChild(createHandLabel("dealerhandl"));
        }
        
        
        
        addSliderBubbles();

    }

            function addNewlineTo(element) {
                element.appendChild(document.createElement("br"));
            }

            function createHandLabel(id) {
                var handValueLabel = document.createElement("span");
                handValueLabel.id = id;
                handValueLabel.innerHTML = "0";
                return handValueLabel;
            }

    function addPlayerControl(playername, isRealPlayer) {
            var playerdiv = document.getElementById(playername + "d");

            function createDisabledButton(id, name) {
                return createDisabledInput(id, name, "button");
            }

            function createDisabledCheckbox(id, name) {
                var seated = createDisabledInput(id, name, "checkbox");
                seated.checked=true;
                return seated;
            }

            function createDisabledInput(id, name, type) {
                var input = document.createElement("input");
                input.id = id;
                input.setAttribute('type', type);
                input.setAttribute('name', name);
                input.setAttribute('value', name);
                input.disabled = true;
                return input;
            }

            function createInputSlider(id, name) {
                var slider = createDisabledInput(id, name, "range")
                slider.disabled = false;
                slider.style.width = "50px";
                slider.min = 1;
                slider.max = 5;
                slider.value = 2;
                return slider;
            }

            function createBetForm(id) {
                var slider = createInputSlider(id, id);
                var form = document.createElement("form");
                form.style.display = "inline";
                var output = document.createElement("output");
                output.setAttribute("for", id);
                //output.setAttribute("onforminput", "value = " + id + ".valueAsNumber;");
               // output.innerHTML = "";
                form.appendChild(slider);
                form.appendChild(output);
                return form;
            }

//            // add seated checkbox
//            addNewlineTo(playerdiv);
//            playerdiv.innerHTML += " Seated:";
//            playerdiv.appendChild(createDisabledCheckbox("player" + j + "seat", "Seated"));

            // add cash label
            if (isRealPlayer) { // dummy player do not need cash and bet label
                addNewlineTo(playerdiv);
                playerdiv.innerHTML += "Cash: $";
                playerdiv.appendChild(createHandLabel(playername + "cash"));
                // and bet amount
                playerdiv.innerHTML += ", Bet: $";
                playerdiv.appendChild(createBetForm(playername + "bet"));
            }

            // hand value label for player
            if (displayHandValue) {
                addNewlineTo(playerdiv);
                playerdiv.innerHTML += "Value: ";
                playerdiv.appendChild(createHandLabel(playername + "handl"));
            }

            // add hit, stand,  double and split buttons
            addNewlineTo(playerdiv);
            playerdiv.appendChild(createDisabledButton(playername + "h", "Hit"));
            playerdiv.appendChild(createDisabledButton(playername + "s", "Stand"));
            playerdiv.appendChild(createDisabledButton(playername + "double", "Double"));
            playerdiv.appendChild(createDisabledButton(playername + "split", "Split"));
    }

        function addPlayerCanvas(styleLeft, styleTop, id) {
            var div = document.createElement( "div" );
            div.className = "semicircle";
            div.style.left = styleLeft;
            div.style.top = styleTop;
            div.id = id + "d";

            // dummy div to draw border
            var borderdiv = document.createElement( "div" );
            borderdiv.className = "borderdiv";
            borderdiv.style.left = "-2px";
            borderdiv.style.top = "-2px";
            div.appendChild(borderdiv);

            // canvas to draw cards on
            var canvas = document.createElement( "canvas" );
            canvas.id = id;
            canvas.width = cardWidth * 5;
            canvas.height = cardHeight;
            div.appendChild(canvas);

            document.getElementById("blackjack-table").appendChild(div);
        }



    // src: http://css-tricks.com/value-bubbles-for-range-inputs/
    function addSliderBubbles() {
        $(function() {
            var el, newPoint, newPlace, offset;
            // Select all range inputs, watch for change
            $("input[type='range']").change(function() {
                // Cache this for efficiency
                el = $(this);
                // Measure width of range input
                width = el.width();
                // Figure out placement percentage between left and right of input
                newPoint = (el.val() - el.attr("min")) / (el.attr("max") - el.attr("min"));
                // Janky value to get pointer to line up better
                offset = -1.3;
                // Prevent bubble from going beyond left or right (unsupported browsers)
                if (newPoint < 0) {
                    newPlace = 0;
                }
                else if (newPoint > 1) {
                    newPlace = width;
                }
                else {
                    newPlace = width * newPoint + offset;
                    offset -= newPoint;
                }
                // Move bubble
                el
                .next("output")
                //                .css({
                //                    left: newPlace,
                //                    marginLeft: offset + "%"
                //                })
                .text(el.val());
            })
            // Fake a change to position bubble at page load
            .trigger('change');
        });
    }

    function initListeners() {
        function addPlayerControlListener(playername) {
                var stand = document.getElementById(playername + "s");
                $(stand).click(function() {
                    standF();
                });
                var hit = document.getElementById(playername + "h");
                $(hit).click(function() {
                    hitF();
                });
                var doubleb = document.getElementById(playername + "double");
                $(doubleb).click(function() {
                    doubleF();
                });
                var split = document.getElementById(playername+ "split");
                $(split).click(function() {
                    splitF();
                });
        }

        for (var j = 0; j < game.player.length; j++) {
            addPlayerControlListener(game.player[j].name);
        }

        $("#NewGame").unbind("click").click(function() {
            newGame();
        });

        $("#NewRound").unbind("click").click(function() {
            newRound();
        });

        // BlackJack Controls (in case buttons are covered by cards)
        $("#Stand").unbind("click").click(function() {
            standF();
        });
        $("#Hit").unbind("click").click(function() {
            hitF();
        });
        $("#Double").unbind("click").click(function() {
            doubleF();
        });
        $("#Split").unbind("click").click(function() {
            splitF();
        });


        $("#DebugView").unbind("click").click(function() {
            drawAllCards();
        });

        $("#DebugView2").unbind("click").click(function() {
            clearHandsDisplayed();
            initView();            
        });

        $("#DebugView3").unbind("click").click(function() {
            showWinnings();
        });

        $("#DebugView4").unbind("click").click(function() {
            var hitb = document.getElementById("player" + 0 + "s");
            //hitb.style.borderColor = "darkred";
           // hitb.style.color = "rgba(139, 0, 0, 0.7)";
           hitb.style.color = "rgba(0, 139, 0, 0.8)";
        });

        $("#HTMLtoConsol").unbind("click").click(function() {
            console.log(document.body.innerHTML);
        });

        // draw all 5 possible cards for each player (debug)
        function drawAllCards() {
            initView();
            initListeners();
            game = new jack.Game(numberOfPlayer);
            game.initRound();
            drawSplit();
            for (var j = 0; j < numberOfPlayer; j++) {
                for (var i = 0; i < 6; i++) {
                    draw(document.getElementById(game.player[j].name).getContext("2d"), new deck.Card(i + 2, deck.ColorEnum.CLUBS), i * cardWidth, 0);
                }
            }
            for (var i = 0; i < 6; i++) {
                draw(document.getElementById("dealer").getContext("2d"), new deck.Card(i + 2, deck.ColorEnum.CLUBS), i * cardWidth, 0);
            }
        }

        function doubleF() {
            dealCardF(jack.Action.DOUBLE);
        }

        function hitF() {
            // disable double
            var doubleb = document.getElementById(game.player[game.currentPlayer].name + "double");
            doubleb.disabled = true;
            // hit
            dealCardF(jack.Action.HIT);
        }

        function standF() {
            var startNewRound = false;
            disableAllPlayerButtons();
            if (!game.playCurrentPlayer(jack.Action.STAND)) { // enable button if there is a next player
                enableCurrentPlayerButtons();
            } else {
                startNewRound = true;
            }
            displayDealtHands(startNewRound); // call with startNewRound to display dealer card value only at end of round

            if (startNewRound) {
                startANewRound();
            }
        }

        function splitF() {
            var startNewRound = false;
            disableAllPlayerButtons();
            game.playCurrentPlayer(jack.Action.SPLIT);
            enableCurrentPlayerButtons();

            // draw view for new player
            var playerDiv = document.getElementById(game.player[game.currentPlayer].belongsTo + "d");
            addPlayerCanvas(playerDiv.style.left, parseInt(playerDiv.style.top) + ((game.getPlayer(game.player[game.currentPlayer].belongsTo).amountChildren) * 200) + "px", game.player[game.currentPlayer + 1].name);
            addPlayerControl(game.player[game.currentPlayer + 1].name, false);
            addPlayerControlListener(game.player[game.currentPlayer + 1].name);

           // clearHandsDisplayed();

           // clear canvas of player that split
           clearCanvas(game.player[game.currentPlayer].name);


            if (animate) {
                animateSplit(game.player[game.currentPlayer], game.player[game.currentPlayer + 1], function() {
                    displayDealtHands(false);
                });
            } else {
                displayDealtHands(false);
            }
        }

        function dealCardF(action) {
            var startNewRound = false;
            var currPlayer = game.currentPlayer; // save value in case playCurrentPlayer changes it
            disableAllPlayerButtons();
            if (!game.playCurrentPlayer(action)) { // enable button if there is a next player
                enableCurrentPlayerButtons();
            } else {
                startNewRound = true;
            }

            if (animate) {
                animateCardDeal(game.player[currPlayer].name, function() {
                    displayDealtHands(startNewRound);
                    if (startNewRound) {
                        startANewRound();
                    }
                });
            } else {
                displayDealtHands(startNewRound);
                    if (startNewRound) {
                        startANewRound();
                    }
            }
        }

    }

    var game;
    newGame();

    /*
     * rebuilds view and starts new round. call when settings have changed.
     */
    function newGame() {
        loadSettings();
        game = new jack.Game(numberOfPlayer);
        initView();
        initListeners();
        
        newRound();
    }

    /**
     * displays some animations and such before calling newRound()
     */
    function startANewRound() {
        showWinnings();
        wait(timeBetweenRounds, function() {newRound();});
    }

        function showWinnings() {
            for (var i = 0; i < game.player.length; i++) {
                var canvas = document.getElementById(game.player[i].name);
                var context = canvas.getContext("2d");
                var w = canvas.width;
                var h = canvas.height;

                context.font = "900 32px Arial";

                // clear background so writing is easier to read:
                var dim = context.measureText(game.player[i].lastWinningsMessage);
                var dimw = dim.width;
                var dimh = context.measureText('M').width; // height apprx
                context.fillStyle = "rgba(255, 255, 255, 0.7)";
                context.strokeStyle = "darkred";
                var offset = 5;
		context.fillRect(10 - offset, ((h / 2) - dimh) - offset, dimw + 3 * offset, dimh + 3 * offset);
                context.strokeRect(10 - offset, ((h / 2) - dimh) - offset, dimw + 3 * offset, dimh + 3 * offset);


                // write message:
                context.fillStyle = "darkred";
                context.fillText(game.player[i].lastWinningsMessage, 10, h / 2);
            }
        }

    /*
     * a function that does nothing for the given amount of milliseconds and
     * then calls the function given in the callback argument.
     */
    function wait(mseconds, callback) {
        setTimeout(function () {
            callback();
        }, mseconds);
    }

    /*
     * starts new round. call when settings have not changed.
     */
    function newRound() {
        removeDummyPlayer();
        game.initRound();
        clearHandsDisplayed();
        updateCashLabel();
        if (animate) {
            animateDealingStartHands(game.player, function() {
                displayDealtHands(false);
            });
        } else {
            displayDealtHands(false);
        }
        setBetsForRound();

        disableAllPlayerButtons();
        enableCurrentPlayerButtons();
    }

    function removeDummyPlayer() {
        function findMatchingElements( toMatch, tagname ) {
            var reMatch = new RegExp( toMatch);
            if ( tagname == null ) tagname = "*"; // if not tagname passed, search all
            var elems = document.getElementsByTagName(tagname);
            var matches = [];
            for ( var e = 0; e < elems.length; ++e ) {
                if ( elems[e].id.match( reMatch )  ) matches.push( elems[e] );
            }
            return matches;
        }
        var dummy = findMatchingElements( 'dummy', 'div' );
        for (var i = 0; i < dummy.length; i++) { 
            dummy[i].parentNode.removeChild(dummy[i]);
        }
    }

    function setBetsForRound() {
        for (var i = 0; i < game.player.length; i++) {
           var betAmount = $("#player" + i + "bet").val();
           betAmount = parseInt(betAmount);
            // TODO sometimes, this is NaN (in firefox and maybe others).
            if (isNaN(betAmount)) {
                betAmount = 3;
            }
           game.player[i].hand.bet = betAmount;
        }
    }

    function updateCashLabel() {
        for (var i = 0; i < game.player.length; i++) {
            var cashLabel = document.getElementById(game.player[i].name + "cash");
            cashLabel.innerHTML = game.player[i].money;
        }
    }

    function clearHandsDisplayed() {
        for (var i = 0; i < game.player.length; i++) {
            clearCanvas(game.player[i].name);
        }
        clearCanvas("dealer");
    }

    function clearCanvas(id) {
        canvas = document.getElementById(id);
        ctx = canvas.getContext("2d");
        ctx.clearRect(0, 0, canvas.width, canvas.height);
    }

    /*
     * draws all cards of all players and the dealer.
     */
    function displayDealtHands(displaydealerHandValue) {
        for (var i = 0; i < game.player.length; i++) {
            var cards = game.player[i].hand.cards.getAllCards();
            for (var j = 0; j < cards.length; j++) {
                draw(document.getElementById(game.player[i].name).getContext("2d"), cards[j] , j * cardWidth, 0);
            }
        }
        var dealercards = game.dealer.hand.cards.getAllCards();
        for (var j = 0; j < dealercards.length; j++) {
            draw(document.getElementById("dealer").getContext("2d"), dealercards[j] , j * cardWidth, 0);
        }

        if (displayHandValue) {
            displayHandValueF(displaydealerHandValue);
        }
    }

    // displays current hand value as text below cards.
    function displayHandValueF(displaydealer) {
        for (var j = 0; j < game.player.length; j++) {
            var handValueLabel = document.getElementById(game.player[j].name + "handl");
            handValueLabel.innerHTML = game.player[j].hand.handValue();
        }
        if (displaydealer) {
            var handValueLabel = document.getElementById("dealerhandl");
            handValueLabel.innerHTML = game.dealer.hand.handValue();
        } else {
            var handValueLabel = document.getElementById("dealerhandl");
            handValueLabel.innerHTML = "?";
        }
        
    }

    // disables all buttons and also resets the font color to black.
    function disableAllPlayerButtons() {
        if (hintbestmove) {
            // reset font color to default:
            for (var j = 0; j < game.player.length; j++) {
                removeClass(game.player[j].name + "s", "highlight");
                removeClass(game.player[j].name + "h", "highlight");
                removeClass(game.player[j].name + "double", "highlight");
                removeClass(game.player[j].name + "split", "highlight");
            }
        }

        // disable all buttons
        for (var j = 0; j < game.player.length; j++) {
            var stand = document.getElementById(game.player[j].name + "s");
            stand.disabled = true;
            var hit = document.getElementById(game.player[j].name + "h");
            hit.disabled = true;
            var doubleb = document.getElementById(game.player[j].name + "double");
            doubleb.disabled = true;
            var split = document.getElementById(game.player[j].name + "split");
            split.disabled = true;
        }
    }

    function enableCurrentPlayerButtons() {
        // enable buttons for current player
        var stand = document.getElementById(game.player[game.currentPlayer].name + "s");
        stand.disabled = false;
        var hit = document.getElementById(game.player[game.currentPlayer].name + "h");
        hit.disabled = false;
        var doubleb = document.getElementById(game.player[game.currentPlayer].name+ "double");
        if (game.player[game.currentPlayer].hand.cards.cardsInDeck() == 2) { // double down on any two cards            
            doubleb.disabled = false;
        }
        var split = document.getElementById(game.player[game.currentPlayer].name + "split");
        if (game.player[game.currentPlayer].hand.cards.cardsInDeck() == 2 // split on any two cards
                    && game.player[game.currentPlayer].hand.isPair() // that are a pair
                    && game.getPlayer(game.player[game.currentPlayer].belongsTo).amountChildren < 3) { // if player has split less than 3 times
            split.disabled = false;
        }

        if (hintbestmove) {
            var hintcolor = "rgba(0, 139, 0, 0.8)";
            var bestMove = getBestMove(game.player[game.currentPlayer].hand, game.dealer.hand);
            switch(bestMove) {
                case jack.Action.HIT:
                    addClass(game.player[game.currentPlayer].name + "h", "highlight");
                    break;
                case jack.Action.STAND:
                    addClass(game.player[game.currentPlayer].name + "s", "highlight");
                    break;
                case jack.Action.DOUBLE:
                    addClass(game.player[game.currentPlayer].name + "double", "highlight");
                    break;
                case jack.Action.SPLIT:
                    addClass(game.player[game.currentPlayer].name + "split", "highlight");
                    break;
                case jack.Action.SURRENDER:
                    //hit.style.color = hintcolor;
                    break;
                default:break;
            }
        }
    }

});
